function energy_norm = E_norm(X, A, N)
% E_NORM  -  Computes the aossciated enery norm
%  Given the operators L(X) = AX + XA^T and P(X) = \sum_{i=1}^m N{i}XN{i}^T
%  If A is negative definite and the operator spectral radius p(L^{-1}P)<1
%  then M(X) = -L(X)-P(X) induces an inner product and thus an energy norm
%  by ||X||_M = trace(X^T M(X))
%  This function computes the energy norm of the operator definied by A and N.
%
%  Obs: It is assumed that the conditions above are fulfiled and that the
%  matrices are real.
%
%  energy_norm = E_norm(X, A, N)
%
%  Input:
%  X - The matrix we are computing the energy norm of
%  A - Matrix defining the Lyapunov operator, should be symmetric and
%    negative definite for a propper norm
%  N - Cell array containing the "generalized" terms of the equation. These
%    terms should be such that

%
%  Output:
%  energy_norm - The associated energy norm
% 

  if ~iscell(N); error('N needs to be a cell-array'); end;
  R = A*X + X*A';
  for i = 1:length(N)
    R = R + (N{i})*X*(N{i}');
  end

  % trace(X^T Y) is the sum of all elementwise products
  energy_norm = sum(sum(X .* R));

end